﻿/***********************************************************
 * 
 * Dance Dance Revolution Mat Library V 1.0
 * Copyright (c) 2009 Michael Hurnaus - michbex@live.com
 * Feel free to use this source for whatever you want. Drop me a line if you found it useful!
 * http://www.michbex.com
 *
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace JoystickInterface
{
    public class DanceDanceRevolutionMatManager
    {
        /// <summary>
        /// This is the device name with which the mat appears in the system.
        /// </summary>
        private const string DDR_MAT_NAME = "USB GamePad";
        
        public delegate void DdrEventHandler(object sender, DdrEventArgs e);
        private int[] buttonMapping = new int[] { 7, 3, 1, 9, 0, 0, 0, 0, 5, 0, 0, 0, 2, 6, 8, 4 };
        
        /// <summary>
        /// List of all Mats, which internally contain keys (and pressed states).
        /// </summary>
        private List<Dictionary<DdrKey, bool>> MatKeys;
        
        private Timer timer;

        public DanceDanceRevolutionMatManager(IntPtr windowHandle)
        {
            JoystickManager.AcquireGamePads(windowHandle, DDR_MAT_NAME);
            NumberOfConnectedDDrMats = JoystickManager.JoystickList.Count;

            MatKeys = new List<Dictionary<DdrKey, bool>>(NumberOfConnectedDDrMats);

            if (NumberOfConnectedDDrMats <= 0)
            {
                throw new InvalidOperationException("No Dancemat found");
            }

            for (int i = 0; i < NumberOfConnectedDDrMats; i++)
            {
                Dictionary<DdrKey, bool> keys = new Dictionary<DdrKey, bool>();
                foreach (DdrKey key in Enum.GetValues(typeof(DdrKey)))
                {
                    keys.Add(key, false);
                }
                MatKeys.Add(keys);
            }            

            timer = new Timer();
            timer.Interval = 100;
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            foreach (Joystick joystick in JoystickManager.JoystickList)
            {
                joystick.UpdateStatus();
                for (int i = 0; i < buttonMapping.Length; i++)
                {
                    int val = buttonMapping[i] + buttonMapping.Length * joystick.ID;
                    if (joystick.Buttons[i])
                    {
                        if (!MatKeys[joystick.ID][(DdrKey)buttonMapping[i]])
                        {
                            MatKeys[joystick.ID][(DdrKey)buttonMapping[i]] = true;
                            OnKeyDown(new DdrEventArgs((DdrKey)buttonMapping[i], joystick.ID));
                        }
                    }
                    else
                    {
                        if (MatKeys[joystick.ID][(DdrKey)buttonMapping[i]])
                        {
                            MatKeys[joystick.ID][(DdrKey)buttonMapping[i]] = false;
                            OnKeyUp(new DdrEventArgs((DdrKey)buttonMapping[i], joystick.ID));
                        }
                    }
                }
            }
        }

        #region EventHandler

        public event DdrEventHandler KeyDown;
        public event DdrEventHandler KeyUp;

        protected virtual void OnKeyDown(DdrEventArgs e)
        {
            if (KeyDown != null)
            {
                KeyDown(this, e);
            }
        }
        protected virtual void OnKeyUp(DdrEventArgs e)
        {
            if (KeyUp != null)
            { 
                KeyUp(this, e);
            }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Number of Dance Dance Revolution Mats connected to the system.
        /// </summary>
        public int NumberOfConnectedDDrMats { get; private set; }

        /// <summary>
        /// Poll frequency for Mat Events in Milliseconds.
        /// </summary>
        public int EventUpdateFrequency
        {
            get
            {
                return timer.Interval;
            }
            set
            {
                timer.Interval = value;
            }
        }

        #endregion
    }

    public class DdrEventArgs : EventArgs
    {
        /// <summary>
        /// The key that was pressed / released.
        /// </summary>
        public DdrKey Key { get; private set; }
        
        /// <summary>
        /// The internal ID of the DDR Mat.
        /// </summary>
        public int DdrId { get; private set; }

        public DdrEventArgs(DdrKey key, int ddrId)
        {
            this.Key = key;
            this.DdrId = ddrId;
        }
    }

    public enum DdrKey
    {
        Start,
        X,
        Up,
        Circle,
        Left,
        Select,
        Right,
        Triangle,
        Down,
        Square
    }
}